// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//TWEAKABLES/////////////////////////////////////////////////

float2 center;

//---------------------

float Fisheye
<float UIMin = 0.1;
float UIMax = 4.0;
float UIStep = 0.02;>
        = 2.0;

//----------------------

float squeeze1
<float UIMin = 1.0;
float UIMax = 3.0;
float UIStep = 0.02;>
        = 1.8;

float squeeze2
<float UIMin = 0.1;
float UIMax = 2.0;
float UIStep = 0.02;>
        = 0.8;

//----------------------

float twirl1
<float UIMin = 0.0;
float UIMax = 2.0;
float UIStep = 0.02;>
        = 0.5;

float twirl2
<float UIMin = 1.0;
float UIMax = 9.0;
float UIStep = 0.10;>
        = 4.0;
        
//------------------------

float bulge1
<float UIMin = -0.5;
float UIMax = 0.5;
float UIStep = 0.02;>
        = -0.1;

float bulge2
<float UIMin = 0.0;
float UIMax = 1.0;
float UIStep = 0.02;>
        = 0.5;

//------------------------

float dent1
<float UIMin = -1.0;
float UIMax = 1.0;
float UIStep = 0.02;>
        = 0.0;

float dent2
<float UIMin = 0.0;
float UIMax = 2.0;
float UIStep = 0.02;>
        = 0.7;

//------------------------

float stretch1
<float UIMin = 0.0;
float UIMax = 0.5;
float UIStep = 0.02;>
        = 0.25;

float stretch2
<float UIMin = 0.0;
float UIMax = 1.0;
float UIStep = 0.02;>
        = 0.5;


//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;


float4x4 tColor <string uiname="Color Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0
    )
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS_Fisheye(vs2ps In): COLOR
{

float2 normCoord = 2.0 * In.TexCd - 1.0 ;  // [-1.0 ,1.0] x [-1.0, 1.0]

normCoord += center;

//Convert to polar coordinatess
float r = length(normCoord);
float phi = atan2(normCoord.y, normCoord.x);

// Effect function: FishEye
r = pow(r, Fisheye) / sqrt(2.0);

// Convert back to cartesian coordinates
normCoord.x = r * cos(phi);
normCoord.y = r * sin(phi);

normCoord -= center;

In.TexCd = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]

float4 col = tex2D(Samp, In.TexCd);

    return mul(col,tColor);
}

//----------------------------------------------------------------------------------------

float4 PS_Squeeze(vs2ps In): COLOR
{

float2 normCoord = 2.0 * In.TexCd - 1.0 ;  // [-1.0 ,1.0] x [-1.0, 1.0]

normCoord += center;

// to polar coordinates
float r = length(normCoord);
float phi = atan2(normCoord.y, normCoord.x);

// Effect function: squeeze
 r = pow(r, 1.0/squeeze1) * squeeze2;

// Convert back to cartesian coordinates
normCoord.x = r * cos(phi);
normCoord.y = r * sin(phi);

normCoord -= center;

In.TexCd = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]

float4 col = tex2D(Samp, In.TexCd);

    return mul(col,tColor);
}

//----------------------------------------------------------------------------------------

float4 PS_Twirl(vs2ps In): COLOR
{

float2 normCoord = 2 * In.TexCd - 1 ;   // [-1.0 ,1.0] x [-1.0, 1.0]

normCoord += center;

// to polar coordinates
float r = length(normCoord);
float phi = atan2(normCoord.y, normCoord.x);

// Effect function: Twirl
phi = phi + (1.0 - smoothstep(-twirl1, twirl1, r)) * twirl2;

 // Convert back to cartesian coordinates
normCoord.x = r * cos(phi);
normCoord.y = r * sin(phi);

normCoord -= center;

In.TexCd = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]

float4 col = tex2D(Samp, In.TexCd);

    return mul(col,tColor);
}

//----------------------------------------------------------------------------------------

float4 PS_Bulge(vs2ps In): COLOR
{

float2 normCoord = 2 * In.TexCd - 1 ; // [-1.0 ,1.0] x [-1.0, 1.0]

normCoord += center;

// to polar coordinates
float r = length(normCoord);
float phi = atan2(normCoord.y, normCoord.x);

// Effect function: Bulge
r = r * smoothstep(bulge1, bulge2, r);

// Convert back to cartesian coordinates
normCoord.x = r * cos(phi);
normCoord.y = r * sin(phi);

normCoord -= center;

In.TexCd = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]

float4 col = tex2D(Samp, In.TexCd);

    return mul(col,tColor);
}

//--------------------------------------------------------------------------------------

float4 PS_Dent(vs2ps In): COLOR
{

float2 normCoord = 2 * In.TexCd - 1 ;   // [-1.0 ,1.0] x [-1.0, 1.0]

normCoord += center;

// to polar coordinates
float r = length(normCoord);
float phi = atan2(normCoord.y, normCoord.x);

// Effect function: Dent
r = 2.0 * r - r * smoothstep(dent1, dent2, r);

// Convert back to cartesian coordinates
normCoord.x = r * cos(phi);
normCoord.y = r * sin(phi);

normCoord -= center;

In.TexCd = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]

float4 col = tex2D(Samp, In.TexCd);

    return mul(col,tColor);
}

//----------------------------------------------------------------------------------

float4 PS_Mirror(vs2ps In): COLOR
{

float2 normCoord = 2 * In.TexCd - 1 ;   // [-1.0 ,1.0] x [-1.0, 1.0]

normCoord += center;

// Effect function: Mirror
normCoord.x = normCoord.x * sign(normCoord.x);

normCoord -= center;

In.TexCd = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]

float4 col = tex2D(Samp, In.TexCd);

    return mul(col,tColor);
}

//-------------------------------------------------------------------------------------

float4 PS_Stretch(vs2ps In): COLOR
{

float2 normCoord = 2 * In.TexCd - 1 ; // [-1.0 ,1.0] x [-1.0, 1.0]

normCoord += center;

// Effect function: Stretch
float2 s = sign(normCoord);
normCoord = abs(normCoord);
normCoord = 0.5 * normCoord + 0.5 * smoothstep(stretch1, stretch2, normCoord + center) * normCoord;
normCoord = s * normCoord;

normCoord -= center;

In.TexCd = normCoord / 2.0 + 0.5; // [0.0 ,1.0] x [0.0, 1.0]

float4 col = tex2D(Samp, In.TexCd);

    return mul(col,tColor);
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Fisheye
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Fisheye();
    }
}

technique Squeeze
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Squeeze();
    }
}

technique Twirl
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Twirl();
    }
}

technique Bulge
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Bulge();
    }
}

technique Dent
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Dent();
    }
}

technique Mirror
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Mirror();
    }
}



technique Stretch
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS_Stretch();
    }
}

/////EOF/////////////////////////////////////////////////////////////////////////////
